try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Role(object):
	def __init__(self):
		"""Creates an instance of Role"""

		self.__display_label = None
		self.__forecast_manager = None
		self.__share_with_peers = None
		self.__name = None
		self.__description = None
		self.__id = None
		self.__reporting_to = None
		self.__admin_user = None
		self.__key_modified = dict()

	def get_display_label(self):
		"""
		The method to get the display_label

		Returns:
			string: A string representing the display_label
		"""

		return self.__display_label

	def set_display_label(self, display_label):
		"""
		The method to set the value to display_label

		Parameters:
			display_label (string) : A string representing the display_label
		"""

		if display_label is not None and not isinstance(display_label, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: display_label EXPECTED TYPE: str', None, None)
		
		self.__display_label = display_label
		self.__key_modified['display_label'] = 1

	def get_forecast_manager(self):
		"""
		The method to get the forecast_manager

		Returns:
			User: An instance of User
		"""

		return self.__forecast_manager

	def set_forecast_manager(self, forecast_manager):
		"""
		The method to set the value to forecast_manager

		Parameters:
			forecast_manager (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if forecast_manager is not None and not isinstance(forecast_manager, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: forecast_manager EXPECTED TYPE: User', None, None)
		
		self.__forecast_manager = forecast_manager
		self.__key_modified['forecast_manager'] = 1

	def get_share_with_peers(self):
		"""
		The method to get the share_with_peers

		Returns:
			bool: A bool representing the share_with_peers
		"""

		return self.__share_with_peers

	def set_share_with_peers(self, share_with_peers):
		"""
		The method to set the value to share_with_peers

		Parameters:
			share_with_peers (bool) : A bool representing the share_with_peers
		"""

		if share_with_peers is not None and not isinstance(share_with_peers, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: share_with_peers EXPECTED TYPE: bool', None, None)
		
		self.__share_with_peers = share_with_peers
		self.__key_modified['share_with_peers'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_description(self):
		"""
		The method to get the description

		Returns:
			string: A string representing the description
		"""

		return self.__description

	def set_description(self, description):
		"""
		The method to set the value to description

		Parameters:
			description (string) : A string representing the description
		"""

		if description is not None and not isinstance(description, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: description EXPECTED TYPE: str', None, None)
		
		self.__description = description
		self.__key_modified['description'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_reporting_to(self):
		"""
		The method to get the reporting_to

		Returns:
			User: An instance of User
		"""

		return self.__reporting_to

	def set_reporting_to(self, reporting_to):
		"""
		The method to set the value to reporting_to

		Parameters:
			reporting_to (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if reporting_to is not None and not isinstance(reporting_to, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: reporting_to EXPECTED TYPE: User', None, None)
		
		self.__reporting_to = reporting_to
		self.__key_modified['reporting_to'] = 1

	def get_admin_user(self):
		"""
		The method to get the admin_user

		Returns:
			bool: A bool representing the admin_user
		"""

		return self.__admin_user

	def set_admin_user(self, admin_user):
		"""
		The method to set the value to admin_user

		Parameters:
			admin_user (bool) : A bool representing the admin_user
		"""

		if admin_user is not None and not isinstance(admin_user, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: admin_user EXPECTED TYPE: bool', None, None)
		
		self.__admin_user = admin_user
		self.__key_modified['admin_user'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class AssociationDetails(object):
	def __init__(self):
		"""Creates an instance of AssociationDetails"""

		self.__lookup_field = None
		self.__related_field = None
		self.__key_modified = dict()

	def get_lookup_field(self):
		"""
		The method to get the lookup_field

		Returns:
			LookupField: An instance of LookupField
		"""

		return self.__lookup_field

	def set_lookup_field(self, lookup_field):
		"""
		The method to set the value to lookup_field

		Parameters:
			lookup_field (LookupField) : An instance of LookupField
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.fields.lookup_field import LookupField
		except Exception:
			from .lookup_field import LookupField

		if lookup_field is not None and not isinstance(lookup_field, LookupField):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: lookup_field EXPECTED TYPE: LookupField', None, None)
		
		self.__lookup_field = lookup_field
		self.__key_modified['lookup_field'] = 1

	def get_related_field(self):
		"""
		The method to get the related_field

		Returns:
			LookupField: An instance of LookupField
		"""

		return self.__related_field

	def set_related_field(self, related_field):
		"""
		The method to set the value to related_field

		Parameters:
			related_field (LookupField) : An instance of LookupField
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.fields.lookup_field import LookupField
		except Exception:
			from .lookup_field import LookupField

		if related_field is not None and not isinstance(related_field, LookupField):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: related_field EXPECTED TYPE: LookupField', None, None)
		
		self.__related_field = related_field
		self.__key_modified['related_field'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.parameter_map import ParameterMap
	from zcrmsdk.src.com.zoho.crm.api.util import APIResponse, CommonAPIHandler, Utility, Constants
	from zcrmsdk.src.com.zoho.crm.api.param import Param
	from zcrmsdk.src.com.zoho.crm.api.header import Header
	from zcrmsdk.src.com.zoho.crm.api.header_map import HeaderMap
except Exception:
	from ..exception import SDKException
	from ..parameter_map import ParameterMap
	from ..util import APIResponse, CommonAPIHandler, Utility, Constants
	from ..param import Param
	from ..header import Header
	from ..header_map import HeaderMap


class RelatedRecordsOperations(object):
	def __init__(self, related_list_api_name, record_id, module_api_name):
		"""
		Creates an instance of RelatedRecordsOperations with the given parameters

		Parameters:
			related_list_api_name (string) : A string representing the related_list_api_name
			record_id (int) : An int representing the record_id
			module_api_name (string) : A string representing the module_api_name
		"""

		if not isinstance(related_list_api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: related_list_api_name EXPECTED TYPE: str', None, None)
		
		if not isinstance(record_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: record_id EXPECTED TYPE: int', None, None)
		
		if not isinstance(module_api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: module_api_name EXPECTED TYPE: str', None, None)
		
		self.__related_list_api_name = related_list_api_name
		self.__record_id = record_id
		self.__module_api_name = module_api_name


	def get_related_records(self, param_instance=None, header_instance=None):
		"""
		The method to get related records

		Parameters:
			param_instance (ParameterMap) : An instance of ParameterMap
			header_instance (HeaderMap) : An instance of HeaderMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if param_instance is not None and not isinstance(param_instance, ParameterMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: param_instance EXPECTED TYPE: ParameterMap', None, None)
		
		if header_instance is not None and not isinstance(header_instance, HeaderMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: header_instance EXPECTED TYPE: HeaderMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2/'
		api_path = api_path + str(self.__module_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(self.__record_id)
		api_path = api_path + '/'
		api_path = api_path + str(self.__related_list_api_name)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.set_param(param_instance)
		handler_instance.set_header(header_instance)
		Utility.get_related_lists(self.__related_list_api_name, self.__module_api_name, handler_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')

	def update_related_records(self, request):
		"""
		The method to update related records

		Parameters:
			request (BodyWrapper) : An instance of BodyWrapper

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.body_wrapper import BodyWrapper
		except Exception:
			from .body_wrapper import BodyWrapper

		if request is not None and not isinstance(request, BodyWrapper):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: request EXPECTED TYPE: BodyWrapper', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2/'
		api_path = api_path + str(self.__module_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(self.__record_id)
		api_path = api_path + '/'
		api_path = api_path + str(self.__related_list_api_name)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_PUT)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_UPDATE)
		handler_instance.set_content_type('application/json')
		handler_instance.set_request(request)
		handler_instance.set_mandatory_checker(True)
		Utility.get_related_lists(self.__related_list_api_name, self.__module_api_name, handler_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')

	def delink_records(self, param_instance=None):
		"""
		The method to delink records

		Parameters:
			param_instance (ParameterMap) : An instance of ParameterMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if param_instance is not None and not isinstance(param_instance, ParameterMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: param_instance EXPECTED TYPE: ParameterMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2/'
		api_path = api_path + str(self.__module_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(self.__record_id)
		api_path = api_path + '/'
		api_path = api_path + str(self.__related_list_api_name)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_DELETE)
		handler_instance.set_category_method(Constants.REQUEST_METHOD_DELETE)
		handler_instance.set_param(param_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')

	def get_related_record(self, related_record_id, header_instance=None):
		"""
		The method to get related record

		Parameters:
			related_record_id (int) : An int representing the related_record_id
			header_instance (HeaderMap) : An instance of HeaderMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if not isinstance(related_record_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: related_record_id EXPECTED TYPE: int', None, None)
		
		if header_instance is not None and not isinstance(header_instance, HeaderMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: header_instance EXPECTED TYPE: HeaderMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2/'
		api_path = api_path + str(self.__module_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(self.__record_id)
		api_path = api_path + '/'
		api_path = api_path + str(self.__related_list_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(related_record_id)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		handler_instance.set_header(header_instance)
		Utility.get_related_lists(self.__related_list_api_name, self.__module_api_name, handler_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')

	def update_related_record(self, related_record_id, request, header_instance=None):
		"""
		The method to update related record

		Parameters:
			related_record_id (int) : An int representing the related_record_id
			request (BodyWrapper) : An instance of BodyWrapper
			header_instance (HeaderMap) : An instance of HeaderMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.body_wrapper import BodyWrapper
		except Exception:
			from .body_wrapper import BodyWrapper

		if not isinstance(related_record_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: related_record_id EXPECTED TYPE: int', None, None)
		
		if request is not None and not isinstance(request, BodyWrapper):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: request EXPECTED TYPE: BodyWrapper', None, None)
		
		if header_instance is not None and not isinstance(header_instance, HeaderMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: header_instance EXPECTED TYPE: HeaderMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2/'
		api_path = api_path + str(self.__module_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(self.__record_id)
		api_path = api_path + '/'
		api_path = api_path + str(self.__related_list_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(related_record_id)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_PUT)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_UPDATE)
		handler_instance.set_content_type('application/json')
		handler_instance.set_request(request)
		handler_instance.set_header(header_instance)
		Utility.get_related_lists(self.__related_list_api_name, self.__module_api_name, handler_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')

	def delink_record(self, related_record_id, header_instance=None):
		"""
		The method to delink record

		Parameters:
			related_record_id (int) : An int representing the related_record_id
			header_instance (HeaderMap) : An instance of HeaderMap

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if not isinstance(related_record_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: related_record_id EXPECTED TYPE: int', None, None)
		
		if header_instance is not None and not isinstance(header_instance, HeaderMap):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: header_instance EXPECTED TYPE: HeaderMap', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/v2/'
		api_path = api_path + str(self.__module_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(self.__record_id)
		api_path = api_path + '/'
		api_path = api_path + str(self.__related_list_api_name)
		api_path = api_path + '/'
		api_path = api_path + str(related_record_id)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_DELETE)
		handler_instance.set_category_method(Constants.REQUEST_METHOD_DELETE)
		handler_instance.set_header(header_instance)
		try:
			from zcrmsdk.src.com.zoho.crm.api.related_records.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')


class GetRelatedRecordsParam(object):
	page = Param('page', 'com.zoho.crm.api.RelatedRecords.GetRelatedRecordsParam')
	per_page = Param('per_page', 'com.zoho.crm.api.RelatedRecords.GetRelatedRecordsParam')


class GetRelatedRecordsHeader(object):
	if_modified_since = Header('If-Modified-Since', 'com.zoho.crm.api.RelatedRecords.GetRelatedRecordsHeader')
	x_external = Header('X-EXTERNAL', 'com.zoho.crm.api.RelatedRecords.GetRelatedRecordsHeader')


class DelinkRecordsParam(object):
	ids = Param('ids', 'com.zoho.crm.api.RelatedRecords.DelinkRecordsParam')


class GetRelatedRecordHeader(object):
	if_modified_since = Header('If-Modified-Since', 'com.zoho.crm.api.RelatedRecords.GetRelatedRecordHeader')


class UpdateRelatedRecordHeader(object):
	x_external = Header('X-EXTERNAL', 'com.zoho.crm.api.RelatedRecords.UpdateRelatedRecordHeader')


class DelinkRecordHeader(object):
	x_external = Header('X-EXTERNAL', 'com.zoho.crm.api.RelatedRecords.DelinkRecordHeader')



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
	from zcrmsdk.src.com.zoho.crm.api.record.record import Record
except Exception:
	from ..exception import SDKException
	from ..util import Constants
	from .record import Record


class Participants(Record):
	def __init__(self):
		"""Creates an instance of Participants"""
		super().__init__()


	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.get_key_value('name')

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.add_key_value('name', name)

	def get_email(self):
		"""
		The method to get the email

		Returns:
			string: A string representing the email
		"""

		return self.get_key_value('Email')

	def set_email(self, email):
		"""
		The method to set the value to email

		Parameters:
			email (string) : A string representing the email
		"""

		if email is not None and not isinstance(email, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: email EXPECTED TYPE: str', None, None)
		
		self.add_key_value('Email', email)

	def get_invited(self):
		"""
		The method to get the invited

		Returns:
			bool: A bool representing the invited
		"""

		return self.get_key_value('invited')

	def set_invited(self, invited):
		"""
		The method to set the value to invited

		Parameters:
			invited (bool) : A bool representing the invited
		"""

		if invited is not None and not isinstance(invited, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: invited EXPECTED TYPE: bool', None, None)
		
		self.add_key_value('invited', invited)

	def get_type(self):
		"""
		The method to get the type

		Returns:
			string: A string representing the type
		"""

		return self.get_key_value('type')

	def set_type(self, type):
		"""
		The method to set the value to type

		Parameters:
			type (string) : A string representing the type
		"""

		if type is not None and not isinstance(type, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: type EXPECTED TYPE: str', None, None)
		
		self.add_key_value('type', type)

	def get_participant(self):
		"""
		The method to get the participant

		Returns:
			string: A string representing the participant
		"""

		return self.get_key_value('participant')

	def set_participant(self, participant):
		"""
		The method to set the value to participant

		Parameters:
			participant (string) : A string representing the participant
		"""

		if participant is not None and not isinstance(participant, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: participant EXPECTED TYPE: str', None, None)
		
		self.add_key_value('participant', participant)

	def get_status(self):
		"""
		The method to get the status

		Returns:
			string: A string representing the status
		"""

		return self.get_key_value('status')

	def set_status(self, status):
		"""
		The method to set the value to status

		Parameters:
			status (string) : A string representing the status
		"""

		if status is not None and not isinstance(status, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: status EXPECTED TYPE: str', None, None)
		
		self.add_key_value('status', status)



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import APIResponse, CommonAPIHandler, Constants
except Exception:
	from ..exception import SDKException
	from ..util import APIResponse, CommonAPIHandler, Constants


class BulkReadOperations(object):
	def __init__(self):
		"""Creates an instance of BulkReadOperations"""
		pass

	def get_bulk_read_job_details(self, job_id):
		"""
		The method to get bulk read job details

		Parameters:
			job_id (int) : An int representing the job_id

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if not isinstance(job_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: job_id EXPECTED TYPE: int', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/bulk/v2/read/'
		api_path = api_path + str(job_id)
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/json')

	def download_result(self, job_id):
		"""
		The method to download result

		Parameters:
			job_id (int) : An int representing the job_id

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		if not isinstance(job_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: job_id EXPECTED TYPE: int', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/bulk/v2/read/'
		api_path = api_path + str(job_id)
		api_path = api_path + '/result'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_GET)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_READ)
		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.response_handler import ResponseHandler
		except Exception:
			from .response_handler import ResponseHandler
		return handler_instance.api_call(ResponseHandler.__module__, 'application/x-download')

	def create_bulk_read_job(self, request):
		"""
		The method to create bulk read job

		Parameters:
			request (RequestWrapper) : An instance of RequestWrapper

		Returns:
			APIResponse: An instance of APIResponse

		Raises:
			SDKException
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.request_wrapper import RequestWrapper
		except Exception:
			from .request_wrapper import RequestWrapper

		if request is not None and not isinstance(request, RequestWrapper):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: request EXPECTED TYPE: RequestWrapper', None, None)
		
		handler_instance = CommonAPIHandler()
		api_path = ''
		api_path = api_path + '/crm/bulk/v2/read'
		handler_instance.set_api_path(api_path)
		handler_instance.set_http_method(Constants.REQUEST_METHOD_POST)
		handler_instance.set_category_method(Constants.REQUEST_CATEGORY_CREATE)
		handler_instance.set_content_type('application/json')
		handler_instance.set_request(request)
		handler_instance.set_mandatory_checker(True)
		try:
			from zcrmsdk.src.com.zoho.crm.api.bulk_read.action_handler import ActionHandler
		except Exception:
			from .action_handler import ActionHandler
		return handler_instance.api_call(ActionHandler.__module__, 'application/json')



from .api_exception import APIException
from .response_handler import ResponseHandler
from .response_wrapper import ResponseWrapper
from .variable_group import VariableGroup
from .variable_groups_operations import VariableGroupsOperations



from .api_exception import APIException
from .response_handler import ResponseHandler
from .profiles_operations import ProfilesOperations, GetProfilesHeader, GetProfileHeader
from .profile import Profile
from .response_wrapper import ResponseWrapper
from .section import Section
from .category import Category
from .permission_detail import PermissionDetail



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Territory(object):
	def __init__(self):
		"""Creates an instance of Territory"""

		self.__manager = None
		self.__name = None
		self.__id = None
		self.__key_modified = dict()

	def get_manager(self):
		"""
		The method to get the manager

		Returns:
			bool: A bool representing the manager
		"""

		return self.__manager

	def set_manager(self, manager):
		"""
		The method to set the value to manager

		Parameters:
			manager (bool) : A bool representing the manager
		"""

		if manager is not None and not isinstance(manager, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: manager EXPECTED TYPE: bool', None, None)
		
		self.__manager = manager
		self.__key_modified['manager'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Profile(object):
	def __init__(self):
		"""Creates an instance of Profile"""

		self.__display_label = None
		self.__created_time = None
		self.__modified_time = None
		self.__permissions_details = None
		self.__name = None
		self.__modified_by = None
		self.__default = None
		self.__description = None
		self.__id = None
		self.__category = None
		self.__created_by = None
		self.__sections = None
		self.__delete = None
		self.__key_modified = dict()

	def get_display_label(self):
		"""
		The method to get the display_label

		Returns:
			string: A string representing the display_label
		"""

		return self.__display_label

	def set_display_label(self, display_label):
		"""
		The method to set the value to display_label

		Parameters:
			display_label (string) : A string representing the display_label
		"""

		if display_label is not None and not isinstance(display_label, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: display_label EXPECTED TYPE: str', None, None)
		
		self.__display_label = display_label
		self.__key_modified['display_label'] = 1

	def get_created_time(self):
		"""
		The method to get the created_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__created_time

	def set_created_time(self, created_time):
		"""
		The method to set the value to created_time

		Parameters:
			created_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if created_time is not None and not isinstance(created_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_time EXPECTED TYPE: datetime', None, None)
		
		self.__created_time = created_time
		self.__key_modified['created_time'] = 1

	def get_modified_time(self):
		"""
		The method to get the modified_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.__modified_time

	def set_modified_time(self, modified_time):
		"""
		The method to set the value to modified_time

		Parameters:
			modified_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if modified_time is not None and not isinstance(modified_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_time EXPECTED TYPE: datetime', None, None)
		
		self.__modified_time = modified_time
		self.__key_modified['modified_time'] = 1

	def get_permissions_details(self):
		"""
		The method to get the permissions_details

		Returns:
			list: An instance of list
		"""

		return self.__permissions_details

	def set_permissions_details(self, permissions_details):
		"""
		The method to set the value to permissions_details

		Parameters:
			permissions_details (list) : An instance of list
		"""

		if permissions_details is not None and not isinstance(permissions_details, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: permissions_details EXPECTED TYPE: list', None, None)
		
		self.__permissions_details = permissions_details
		self.__key_modified['permissions_details'] = 1

	def get_name(self):
		"""
		The method to get the name

		Returns:
			string: A string representing the name
		"""

		return self.__name

	def set_name(self, name):
		"""
		The method to set the value to name

		Parameters:
			name (string) : A string representing the name
		"""

		if name is not None and not isinstance(name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: name EXPECTED TYPE: str', None, None)
		
		self.__name = name
		self.__key_modified['name'] = 1

	def get_modified_by(self):
		"""
		The method to get the modified_by

		Returns:
			User: An instance of User
		"""

		return self.__modified_by

	def set_modified_by(self, modified_by):
		"""
		The method to set the value to modified_by

		Parameters:
			modified_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if modified_by is not None and not isinstance(modified_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modified_by EXPECTED TYPE: User', None, None)
		
		self.__modified_by = modified_by
		self.__key_modified['modified_by'] = 1

	def get_default(self):
		"""
		The method to get the default

		Returns:
			bool: A bool representing the default
		"""

		return self.__default

	def set_default(self, default):
		"""
		The method to set the value to default

		Parameters:
			default (bool) : A bool representing the default
		"""

		if default is not None and not isinstance(default, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: default EXPECTED TYPE: bool', None, None)
		
		self.__default = default
		self.__key_modified['default'] = 1

	def get_description(self):
		"""
		The method to get the description

		Returns:
			string: A string representing the description
		"""

		return self.__description

	def set_description(self, description):
		"""
		The method to set the value to description

		Parameters:
			description (string) : A string representing the description
		"""

		if description is not None and not isinstance(description, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: description EXPECTED TYPE: str', None, None)
		
		self.__description = description
		self.__key_modified['description'] = 1

	def get_id(self):
		"""
		The method to get the id

		Returns:
			int: An int representing the id
		"""

		return self.__id

	def set_id(self, id):
		"""
		The method to set the value to id

		Parameters:
			id (int) : An int representing the id
		"""

		if id is not None and not isinstance(id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: id EXPECTED TYPE: int', None, None)
		
		self.__id = id
		self.__key_modified['id'] = 1

	def get_category(self):
		"""
		The method to get the category

		Returns:
			bool: A bool representing the category
		"""

		return self.__category

	def set_category(self, category):
		"""
		The method to set the value to category

		Parameters:
			category (bool) : A bool representing the category
		"""

		if category is not None and not isinstance(category, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: category EXPECTED TYPE: bool', None, None)
		
		self.__category = category
		self.__key_modified['category'] = 1

	def get_created_by(self):
		"""
		The method to get the created_by

		Returns:
			User: An instance of User
		"""

		return self.__created_by

	def set_created_by(self, created_by):
		"""
		The method to set the value to created_by

		Parameters:
			created_by (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if created_by is not None and not isinstance(created_by, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: created_by EXPECTED TYPE: User', None, None)
		
		self.__created_by = created_by
		self.__key_modified['created_by'] = 1

	def get_sections(self):
		"""
		The method to get the sections

		Returns:
			list: An instance of list
		"""

		return self.__sections

	def set_sections(self, sections):
		"""
		The method to set the value to sections

		Parameters:
			sections (list) : An instance of list
		"""

		if sections is not None and not isinstance(sections, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: sections EXPECTED TYPE: list', None, None)
		
		self.__sections = sections
		self.__key_modified['sections'] = 1

	def get_delete(self):
		"""
		The method to get the delete

		Returns:
			bool: A bool representing the delete
		"""

		return self.__delete

	def set_delete(self, delete):
		"""
		The method to set the value to delete

		Parameters:
			delete (bool) : A bool representing the delete
		"""

		if delete is not None and not isinstance(delete, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: delete EXPECTED TYPE: bool', None, None)
		
		self.__delete = delete
		self.__key_modified['_delete'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



from .deleted_record import DeletedRecord
from .criteria import Criteria
from .file_details import FileDetails
from .pricing_details import PricingDetails
from .record_operations import RecordOperations, GetRecordParam, GetRecordHeader, UpdateRecordHeader, DeleteRecordParam, DeleteRecordHeader, GetRecordsParam, GetRecordsHeader, UpdateRecordsHeader, DeleteRecordsParam, DeleteRecordsHeader, UpsertRecordsHeader, GetDeletedRecordsParam, GetDeletedRecordsHeader, SearchRecordsParam, SearchRecordsHeader, GetMassUpdateStatusParam
from .api_exception import APIException
from .response_handler import ResponseHandler
from .action_response import ActionResponse
from .mass_update_response_wrapper import MassUpdateResponseWrapper
from .success_response import SuccessResponse
from .file_body_wrapper import FileBodyWrapper
from .recurring_activity import RecurringActivity
from .record import Record
from .mass_update_response import MassUpdateResponse
from .options import Options
from .action_handler import ActionHandler
from .download_handler import DownloadHandler
from .info import Info
from .participants import Participants
from .convert_action_response import ConvertActionResponse
from .successful_convert import SuccessfulConvert
from .action_wrapper import ActionWrapper
from .mass_update_action_wrapper import MassUpdateActionWrapper
from .reminder import Reminder
from .mass_update_action_handler import MassUpdateActionHandler
from .mass_update_body_wrapper import MassUpdateBodyWrapper
from .convert_action_wrapper import ConvertActionWrapper
from .deleted_records_wrapper import DeletedRecordsWrapper
from .body_wrapper import BodyWrapper
from .consent import Consent
from .remind_at import RemindAt
from .mass_update_action_response import MassUpdateActionResponse
from .file_handler import FileHandler
from .lead_converter import LeadConverter
from .convert_body_wrapper import ConvertBodyWrapper
from .deleted_records_handler import DeletedRecordsHandler
from .mass_update_response_handler import MassUpdateResponseHandler
from .inventory_line_items import InventoryLineItems
from .line_item_product import LineItemProduct
from .field import Field
from .carry_over_tags import CarryOverTags
from .response_wrapper import ResponseWrapper
from .comment import Comment
from .mass_update_success_response import MassUpdateSuccessResponse
from .convert_action_handler import ConvertActionHandler
from .mass_update import MassUpdate
from .line_tax import LineTax
from .territory import Territory



from .action_wrapper import ActionWrapper
from .request_wrapper import RequestWrapper
from .criteria import Criteria
from .query import Query
from .result import Result
from .job_detail import JobDetail
from .api_exception import APIException
from .action_response import ActionResponse
from .response_handler import ResponseHandler
from .bulk_read_operations import BulkReadOperations
from .success_response import SuccessResponse
from .call_back import CallBack
from .file_body_wrapper import FileBodyWrapper
from .response_wrapper import ResponseWrapper
from .action_handler import ActionHandler



from .action_wrapper import ActionWrapper
from .merge_wrapper import MergeWrapper
from .record_action_wrapper import RecordActionWrapper
from .body_wrapper import BodyWrapper
from .tags_operations import TagsOperations, GetTagsParam, CreateTagsParam, UpdateTagsParam, UpdateTagParam, AddTagsToRecordParam, RemoveTagsFromRecordParam, AddTagsToMultipleRecordsParam, RemoveTagsFromMultipleRecordsParam, GetRecordCountForTagParam
from .record_action_handler import RecordActionHandler
from .api_exception import APIException
from .count_wrapper import CountWrapper
from .action_response import ActionResponse
from .response_handler import ResponseHandler
from .success_response import SuccessResponse
from .conflict_wrapper import ConflictWrapper
from .response_wrapper import ResponseWrapper
from .count_handler import CountHandler
from .action_handler import ActionHandler
from .record_action_response import RecordActionResponse
from .tag import Tag
from .info import Info



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Notification(object):
	def __init__(self):
		"""Creates an instance of Notification"""

		self.__channel_expiry = None
		self.__resource_uri = None
		self.__resource_id = None
		self.__notify_url = None
		self.__resource_name = None
		self.__channel_id = None
		self.__events = None
		self.__token = None
		self.__notify_on_related_action = None
		self.__fields = None
		self.__deleteevents = None
		self.__key_modified = dict()

	def get_channel_expiry(self):
		"""
		The method to get the channel_expiry

		Returns:
			datetime: An instance of datetime
		"""

		return self.__channel_expiry

	def set_channel_expiry(self, channel_expiry):
		"""
		The method to set the value to channel_expiry

		Parameters:
			channel_expiry (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if channel_expiry is not None and not isinstance(channel_expiry, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: channel_expiry EXPECTED TYPE: datetime', None, None)
		
		self.__channel_expiry = channel_expiry
		self.__key_modified['channel_expiry'] = 1

	def get_resource_uri(self):
		"""
		The method to get the resource_uri

		Returns:
			string: A string representing the resource_uri
		"""

		return self.__resource_uri

	def set_resource_uri(self, resource_uri):
		"""
		The method to set the value to resource_uri

		Parameters:
			resource_uri (string) : A string representing the resource_uri
		"""

		if resource_uri is not None and not isinstance(resource_uri, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: resource_uri EXPECTED TYPE: str', None, None)
		
		self.__resource_uri = resource_uri
		self.__key_modified['resource_uri'] = 1

	def get_resource_id(self):
		"""
		The method to get the resource_id

		Returns:
			string: A string representing the resource_id
		"""

		return self.__resource_id

	def set_resource_id(self, resource_id):
		"""
		The method to set the value to resource_id

		Parameters:
			resource_id (string) : A string representing the resource_id
		"""

		if resource_id is not None and not isinstance(resource_id, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: resource_id EXPECTED TYPE: str', None, None)
		
		self.__resource_id = resource_id
		self.__key_modified['resource_id'] = 1

	def get_notify_url(self):
		"""
		The method to get the notify_url

		Returns:
			string: A string representing the notify_url
		"""

		return self.__notify_url

	def set_notify_url(self, notify_url):
		"""
		The method to set the value to notify_url

		Parameters:
			notify_url (string) : A string representing the notify_url
		"""

		if notify_url is not None and not isinstance(notify_url, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: notify_url EXPECTED TYPE: str', None, None)
		
		self.__notify_url = notify_url
		self.__key_modified['notify_url'] = 1

	def get_resource_name(self):
		"""
		The method to get the resource_name

		Returns:
			string: A string representing the resource_name
		"""

		return self.__resource_name

	def set_resource_name(self, resource_name):
		"""
		The method to set the value to resource_name

		Parameters:
			resource_name (string) : A string representing the resource_name
		"""

		if resource_name is not None and not isinstance(resource_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: resource_name EXPECTED TYPE: str', None, None)
		
		self.__resource_name = resource_name
		self.__key_modified['resource_name'] = 1

	def get_channel_id(self):
		"""
		The method to get the channel_id

		Returns:
			int: An int representing the channel_id
		"""

		return self.__channel_id

	def set_channel_id(self, channel_id):
		"""
		The method to set the value to channel_id

		Parameters:
			channel_id (int) : An int representing the channel_id
		"""

		if channel_id is not None and not isinstance(channel_id, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: channel_id EXPECTED TYPE: int', None, None)
		
		self.__channel_id = channel_id
		self.__key_modified['channel_id'] = 1

	def get_events(self):
		"""
		The method to get the events

		Returns:
			list: An instance of list
		"""

		return self.__events

	def set_events(self, events):
		"""
		The method to set the value to events

		Parameters:
			events (list) : An instance of list
		"""

		if events is not None and not isinstance(events, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: events EXPECTED TYPE: list', None, None)
		
		self.__events = events
		self.__key_modified['events'] = 1

	def get_token(self):
		"""
		The method to get the token

		Returns:
			string: A string representing the token
		"""

		return self.__token

	def set_token(self, token):
		"""
		The method to set the value to token

		Parameters:
			token (string) : A string representing the token
		"""

		if token is not None and not isinstance(token, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: token EXPECTED TYPE: str', None, None)
		
		self.__token = token
		self.__key_modified['token'] = 1

	def get_notify_on_related_action(self):
		"""
		The method to get the notify_on_related_action

		Returns:
			bool: A bool representing the notify_on_related_action
		"""

		return self.__notify_on_related_action

	def set_notify_on_related_action(self, notify_on_related_action):
		"""
		The method to set the value to notify_on_related_action

		Parameters:
			notify_on_related_action (bool) : A bool representing the notify_on_related_action
		"""

		if notify_on_related_action is not None and not isinstance(notify_on_related_action, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: notify_on_related_action EXPECTED TYPE: bool', None, None)
		
		self.__notify_on_related_action = notify_on_related_action
		self.__key_modified['notify_on_related_action'] = 1

	def get_fields(self):
		"""
		The method to get the fields

		Returns:
			dict: An instance of dict
		"""

		return self.__fields

	def set_fields(self, fields):
		"""
		The method to set the value to fields

		Parameters:
			fields (dict) : An instance of dict
		"""

		if fields is not None and not isinstance(fields, dict):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: fields EXPECTED TYPE: dict', None, None)
		
		self.__fields = fields
		self.__key_modified['fields'] = 1

	def get_deleteevents(self):
		"""
		The method to get the deleteevents

		Returns:
			bool: A bool representing the deleteevents
		"""

		return self.__deleteevents

	def set_deleteevents(self, deleteevents):
		"""
		The method to set the value to deleteevents

		Parameters:
			deleteevents (bool) : A bool representing the deleteevents
		"""

		if deleteevents is not None and not isinstance(deleteevents, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: deleteevents EXPECTED TYPE: bool', None, None)
		
		self.__deleteevents = deleteevents
		self.__key_modified['_delete_events'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



import datetime


class Constants(object):

    """
    This class uses the SDK constants name reference.
    """

    DATA_TYPE_ERROR = "DATA TYPE ERROR"

    ERROR = "error"

    REQUEST_METHOD_GET = "GET"

    REQUEST_METHOD_POST = "POST"

    REQUEST_METHOD_PUT = "PUT"

    REQUEST_METHOD_DELETE = "DELETE"

    REQUEST_METHOD_PATCH = 'PATCH'

    OAUTH_HEADER_PREFIX = "Zoho-oauthtoken "

    AUTHORIZATION = "Authorization"

    API_NAME = "api_name"

    INVALID_ID_MSG = "The given id seems to be invalid."

    API_MAX_RECORDS_MSG = "Cannot process more than 100 records at a time."

    INVALID_DATA = "INVALID_DATA"

    CODE_SUCCESS = "SUCCESS"

    STATUS_SUCCESS = "success"

    STATUS_ERROR = "error"

    GRANT_TYPE = "grant_type"

    GRANT_TYPE_AUTH_CODE = "authorization_code"

    ACCESS_TOKEN = "access_token"

    EXPIRES_IN = "expires_in"

    EXPIRES_IN_SEC = "expires_in_sec"

    REFRESH_TOKEN = "refresh_token"

    CLIENT_ID = "client_id"

    CLIENT_SECRET = "client_secret"

    REDIRECT_URL = "redirect_uri"

    TYPE_VS_DATATYPE = {
        "String": str,
        "List": list,
        "Integer": int,
        "HashMap": dict,
        "Map": dict,
        "Long": int,
        "Double": float,
        "Float": float,
        "DateTime": datetime.datetime,
        "Date": datetime.date,
        "Boolean": bool
    }

    OBJECT_KEY = "object"

    GIVEN_LENGTH = "given-length"

    ZOHO_SDK = "X-ZOHO-SDK"

    SDK_VERSION = "3.1.0"

    CONTENT_DISPOSITION = "Content-Disposition"

    TOKEN_ERROR = "TOKEN ERROR"

    SAVE_TOKEN_ERROR = "Exception in saving tokens"

    INVALID_CLIENT_ERROR = "INVALID CLIENT ERROR"

    ERROR_KEY = 'error'

    GET_TOKEN_ERROR = "Exception in getting access token"

    MYSQL_HOST = "localhost"

    MYSQL_DATABASE_NAME = "zohooauth"

    MYSQL_USER_NAME = "root"

    MYSQL_PORT_NUMBER = "3306"

    GET_TOKEN_DB_ERROR = "Exception in getToken - DBStore"

    TOKEN_STORE = "TOKEN_STORE"

    DELETE_TOKEN_DB_ERROR = "Exception in delete_token - DBStore"

    SAVE_TOKEN_DB_ERROR = "Exception in save_token - DBStore"

    USER_MAIL = "user_mail"

    GRANT_TOKEN = "grant_token"

    EXPIRY_TIME = "expiry_time"

    GET_TOKEN_FILE_ERROR = "Exception in get_token - FileStore"

    SAVE_TOKEN_FILE_ERROR = "Exception in save_token - FileStore"

    DELETE_TOKEN_FILE_ERROR = "Exception in delete_token - FileStore"

    TYPE = "type"

    STREAM_WRAPPER_CLASS_PATH = 'zcrmsdk.src.com.zoho.crm.api.util.StreamWrapper'

    FIELD = "field"

    NAME = "name"

    INDEX = "index"

    CLASS = "class"

    ACCEPTED_TYPE = "accepted_type"

    GIVEN_TYPE = "given_type"

    TYPE_ERROR = "TYPE ERROR"

    VALUES = "values"

    ACCEPTED_VALUES = "accepted_values"

    UNACCEPTED_VALUES_ERROR = "UNACCEPTED VALUES ERROR"

    MIN_LENGTH = "min-length"

    MINIMUM_LENGTH = "minimum-length"

    MINIMUM_LENGTH_ERROR = "MINIMUM LENGTH ERROR"

    UNIQUE = "unique"

    FIRST_INDEX = "first-index"

    NEXT_INDEX = "next-index"

    UNIQUE_KEY_ERROR = "UNIQUE KEY ERROR"

    MAX_LENGTH = "max-length"

    MAXIMUM_LENGTH = "maximum-length"

    MAXIMUM_LENGTH_ERROR = "MAXIMUM LENGTH ERROR"

    REGEX = "regex"

    INSTANCE_NUMBER = "instance-number"

    REGEX_MISMATCH_ERROR = "REGEX MISMATCH ERROR"

    READ_ONLY = "read-only"

    IS_KEY_MODIFIED = 'is_key_modified'

    REQUIRED = "required"

    REQUIRED_IN_UPDATE = "required_in_update"

    PRIMARY = "primary"

    MANDATORY_VALUE_ERROR = "MANDATORY VALUE ERROR"

    MANDATORY_VALUE_NULL_ERROR = "MANDATORY VALUE NULL ERROR"

    MANDATORY_KEY_ERROR = "Value missing or None for mandatory key(s): "

    MANDATORY_KEY_NULL_ERROR = "Null Value for mandatory key : "

    SET_KEY_MODIFIED = "set_key_modified"

    LIST_NAMESPACE = "List"

    MAP_NAMESPACE = "Map"

    HASH_MAP = "HashMap"

    STRUCTURE_NAME = "structure_name"

    KEYS = "keys"

    INTERFACE = "interface"

    RECORD_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.Record"

    RECORD_TYPE = "zcrmsdk.src.com.zoho.crm.api.record.record.Record"

    INVENTORY_LINE_ITEMS = "zcrmsdk.src.com.zoho.crm.api.record.InventoryLineItems"

    PRICINGDETAILS = "zcrmsdk.src.com.zoho.crm.api.record.PricingDetails"

    COMMENT_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.Comment"

    FIELD_FILE_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.FileDetails"

    REMINDAT_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.RemindAt"

    CONSENT_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.Consent"

    LINE_TAX_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.LineTax"

    REMINDER_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.Reminder"

    ATTACHMENTS_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.attachments.Attachment"

    RECURRING_ACTIVITY_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.record.RecurringActivity"

    PARTICIPANTS = "zcrmsdk.src.com.zoho.crm.api.record.Participants"

    PACKAGE_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api."

    USER_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.users.User"

    MODULE_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.modules.Module"

    LAYOUT_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.layouts.Layout"

    CHOICE_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.util.Choice"

    KEY_VALUES = "key_values"

    RECORD_CLASS_NAME = "Record"

    KEY_MODIFIED = "key_modified"

    COMMENTS = "Comments"

    CLASSES = "classes"

    LOGFILE_NAME = "sdk_logs.log"

    USER = "user"

    EXPECTED_TYPE = "expected-type"

    INITIALIZATION_ERROR = "INITIALIZATION ERROR"

    ENVIRONMENT = "environment"

    STORE = "store"

    TOKEN = "token"

    SDK_CONFIG = "sdk_config"

    USER_PROXY = "proxy"

    JSON_DETAILS_FILE_PATH = 'json_details.json'

    EMAIL = "email"

    USER_SIGNATURE_ERROR = "USERSIGNATURE ERROR";

    USER_INITIALIZATION_ERROR = "Error during User Initialization"

    EMAIL_REGEX = '^[_a-z0-9-]+(.[_a-z0-9-]+)*@[a-z0-9-]+(.[a-z0-9-]+)*(.[a-z]{2,4})$'

    STRING = "String"

    TOKEN_TYPE = "token_type"

    GRANT = "GRANT"

    REFRESH = "REFRESH"

    CONTENT_TYPE = 'Content-Type'

    STRING_NAMESPACE = 'String'

    BOOLEAN_NAMESPACE = 'Boolean'

    INTEGER_NAMESPACE = 'Integer'

    LONG_NAMESPACE = 'String'

    DOUBLE_NAMESPACE = 'Float'

    DATE_NAMESPACE = 'Date'

    DATETIME_NAMESPACE = 'DateTime'

    FILE_NAMESPACE = "zcrmsdk.src.com.zoho.crm.api.util.StreamWrapper"

    KEYS_TO_SKIP = ['Created_Time', 'Modified_Time', 'Created_By', 'Modified_By', 'Tag']

    PRODUCT_DETAILS = "Product_Details"

    PRICING_DETAILS = "Pricing_Details"

    PARTICIPANT_API_NAME = "Participants"

    INVENTORY_MODULES = ['invoices', 'sales_orders', 'purchase_orders', 'quotes']

    PRICE_BOOKS = "price_books"

    EVENTS = "events"

    CALLS = "calls"

    CALL_DURATION = "call_duration"

    SOLUTIONS = "solutions"

    CASES = "cases"

    ACTIVITIES = "activities"

    LAYOUT = "Layout"

    SUBFORM = "subform"

    LOOKUP = "lookup"

    SE_MODULE = "se_module"

    MODULEDETAILS = 'moduleDetails'

    MODULEPACKAGENAME = "modulePackageName"

    UNDERSCORE = "_"

    RELATED_LISTS = "Related_Lists"

    FIELD_DETAILS_DIRECTORY = "resources"

    NO_CONTENT_STATUS_CODE = 204

    NOT_MODIFIED_STATUS_CODE = 304

    HREF = "href"

    API_EXCEPTION = "API_EXCEPTION"

    FIELDS_LAST_MODIFIED_TIME = "FIELDS-LAST-MODIFIED-TIME"

    LINE_TAX = "$line_tax"

    CANT_DISCLOSE = " ## can't disclose ## "

    URL = "URL"

    HEADERS = "HEADERS"

    PARAMS = "PARAMS"

    INITIALIZATION_SUCCESSFUL = "Initialization successful "

    INITIALIZATION_SWITCHED = "Initialization switched "

    IN_ENVIRONMENT = " in Environment : "

    FOR_EMAIL_ID = "for Email Id : "

    REFRESH_TOKEN_MESSAGE = "Access Token has expired. Hence refreshing."

    RESOURCE_PATH_ERROR = "EMPTY_RESOURCE_PATH"

    RESOURCE_PATH_ERROR_MESSAGE = "Resource Path MUST NOT be None/empty."

    SET_TO_CONTENT_TYPE = ["/crm/bulk/v2/read", "/crm/bulk/v2/write"]

    CONTENT_TYPE_HEADER = "Content-Type"

    LIST_KEY = "List"

    ATTACHMENT_ID = "attachment_id"

    FILE_ID = "file_id"

    DELETE_FIELD_FILE_ERROR = "Exception in deleting Current User Fields file : "

    DELETE_FIELD_FILES_ERROR = "Exception in deleting all Fields files : "

    DELETE_MODULE_FROM_FIELDFILE_ERROR = "Exception in deleting module from Fields file : "

    FILE_BODY_WRAPPER = 'FileBodyWrapper'

    EXCEPTION_IS_KEY_MODIFIED = "Exception in calling is_key_modified : "

    EXCEPTION_SET_KEY_MODIFIED = "Exception in calling set_key_modified : "

    SET_API_URL_EXCEPTION = "Exception in setting API URL : "

    AUTHENTICATION_EXCEPTION = "Exception in authenticating current request : "

    FORM_REQUEST_EXCEPTION = "Exception in forming request body : "

    API_CALL_EXCEPTION = "Exception in current API call execution : "

    HTTP = "http"

    HTTPS = "https"

    CONTENT_API_URL = "content.zohoapis.com"

    EXCEPTION = "Exception"

    MODULE = "module"

    REQUEST_CATEGORY_READ = "READ"

    REQUEST_CATEGORY_CREATE = "CREATE"

    REQUEST_CATEGORY_UPDATE = "UPDATE"

    ID = "id"

    CODE = "code"

    STATUS = "status"

    MESSAGE = "message"

    INVALID_URL_ERROR = "Invalid URL Error"

    REQUEST_CATEGORY_ACTION = "ACTION"

    SKIP_MANDATORY = "skip_mandatory"

    FORMULA = "formula"

    PICKLIST = "picklist"

    ACCOUNTS = "accounts"

    PRIMARY_KEY_ERROR = "Value missing or None for required key(s) : "

    REFRESH_SINGLE_MODULE_FIELDS_ERROR = "Exception in refreshing fields of module : "

    REFRESH_ALL_MODULE_FIELDS_ERROR = "Exception in refreshing fields of all modules : "

    GET_TOKENS_DB_ERROR = "Exception in get_tokens - DBStore"

    DELETE_TOKENS_DB_ERROR = "Exception in delete_tokens - DBStore"

    GET_TOKENS_FILE_ERROR = "Exception in get_tokens - FileStore"

    DELETE_TOKENS_FILE_ERROR = "Exception in delete_tokens - FileStore"

    SDK_UNINITIALIZATION_ERROR = "SDK UNINITIALIZED ERROR"

    SDK_UNINITIALIZATION_MESSAGE = "SDK is UnInitialized"

    INITIALIZATION_EXCEPTION = "Exception in initialization"

    SWITCH_USER_EXCEPTION = "Exception in switching user"

    AUTO_REFRESH_FIELDS_ERROR_MESSAGE = "auto_refresh_fields MUST NOT be None."

    HEADER_NONE_ERROR = "NONE HEADER ERROR"

    HEADER_INSTANCE_NONE_ERROR = "Header Instance MUST NOT be None"

    PARAM_INSTANCE_NONE_ERROR = "Param Instance MUST NOT be None"

    HEADER_NAME_NONE_ERROR = "NONE HEADER NAME ERROR"

    HEADER_NAME_NULL_ERROR_MESSAGE = "Header Name MUST NOT be null"

    NONE_VALUE_ERROR_MESSAGE = " MUST NOT be None"

    PARAMETER_NONE_ERROR = "NONE PARAMETER ERROR"

    PARAM_NAME_NONE_ERROR = "NONE PARAM NAME ERROR"

    PARAM_NAME_NONE_ERROR_MESSAGE = "Param Name MUST NOT be None"

    SWITCH_USER_ERROR = "SWITCH USER ERROR"

    USER_PROXY_ERROR = "USERPROXY ERROR"

    HOST_ERROR_MESSAGE = "Host MUST NOT be None"

    PORT_ERROR_MESSAGE = "Port MUST NOT be None"

    USER_ERROR_MESSAGE = "User MUST NOT be None/empty"

    UNSUPPORTED_IN_API = "API UNSUPPORTED OPERATION"

    UNSUPPORTED_IN_API_MESSAGE = " Operation is not supported by API"

    NULL_VALUE = "null"

    NOTES = "notes"

    ATTACHMENTS = "$attachments"

    PROXY_SETTINGS = "Proxy settings - "

    PROXY_HOST = "Host: "

    PROXY_PORT = "Port: "

    PROXY_USER = "User: "

    JSON_FILE_EXTENSION = ".json"

    FILE_ERROR = "file_error"

    FILE_DOES_NOT_EXISTS = "file does not exists"

    CONSENT_LOOKUP = "consent_lookup"

    USER_MAIL_NULL_ERROR = "USER MAIL NONE ERROR"

    USER_MAIL_NULL_ERROR_MESSAGE = "User Mail MUST NOT be None. Set value to user_mail."

    JSON_DETAILS_ERROR = "ERROR IN READING JSONDETAILS FILE"



from .action_wrapper import ActionWrapper
from .share_record import ShareRecord
from .delete_action_handler import DeleteActionHandler
from .shared_through import SharedThrough
from .body_wrapper import BodyWrapper
from .api_exception import APIException
from .share_records_operations import ShareRecordsOperations, GetSharedRecordDetailsParam
from .response_handler import ResponseHandler
from .action_response import ActionResponse
from .success_response import SuccessResponse
from .response_wrapper import ResponseWrapper
from .action_handler import ActionHandler
from .delete_action_wrapper import DeleteActionWrapper
from .delete_action_response import DeleteActionResponse



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
	from zcrmsdk.src.com.zoho.crm.api.share_records.response_handler import ResponseHandler
except Exception:
	from ..exception import SDKException
	from ..util import Constants
	from .response_handler import ResponseHandler


class ResponseWrapper(ResponseHandler):
	def __init__(self):
		"""Creates an instance of ResponseWrapper"""
		super().__init__()

		self.__share = None
		self.__shareable_user = None
		self.__key_modified = dict()

	def get_share(self):
		"""
		The method to get the share

		Returns:
			list: An instance of list
		"""

		return self.__share

	def set_share(self, share):
		"""
		The method to set the value to share

		Parameters:
			share (list) : An instance of list
		"""

		if share is not None and not isinstance(share, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: share EXPECTED TYPE: list', None, None)
		
		self.__share = share
		self.__key_modified['share'] = 1

	def get_shareable_user(self):
		"""
		The method to get the shareable_user

		Returns:
			list: An instance of list
		"""

		return self.__shareable_user

	def set_shareable_user(self, shareable_user):
		"""
		The method to set the value to shareable_user

		Parameters:
			shareable_user (list) : An instance of list
		"""

		if shareable_user is not None and not isinstance(shareable_user, list):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: shareable_user EXPECTED TYPE: list', None, None)
		
		self.__shareable_user = shareable_user
		self.__key_modified['shareable_user'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class TabTheme(object):
	def __init__(self):
		"""Creates an instance of TabTheme"""

		self.__font_color = None
		self.__background = None
		self.__key_modified = dict()

	def get_font_color(self):
		"""
		The method to get the font_color

		Returns:
			string: A string representing the font_color
		"""

		return self.__font_color

	def set_font_color(self, font_color):
		"""
		The method to set the value to font_color

		Parameters:
			font_color (string) : A string representing the font_color
		"""

		if font_color is not None and not isinstance(font_color, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: font_color EXPECTED TYPE: str', None, None)
		
		self.__font_color = font_color
		self.__key_modified['font_color'] = 1

	def get_background(self):
		"""
		The method to get the background

		Returns:
			string: A string representing the background
		"""

		return self.__background

	def set_background(self, background):
		"""
		The method to set the value to background

		Parameters:
			background (string) : A string representing the background
		"""

		if background is not None and not isinstance(background, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: background EXPECTED TYPE: str', None, None)
		
		self.__background = background
		self.__key_modified['background'] = 1

	def is_key_modified(self, key):
		"""
		The method to check if the user has modified the given key

		Parameters:
			key (string) : A string representing the key

		Returns:
			int: An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if key in self.__key_modified:
			return self.__key_modified.get(key)
		
		return None

	def set_key_modified(self, key, modification):
		"""
		The method to mark the given key as modified

		Parameters:
			key (string) : A string representing the key
			modification (int) : An int representing the modification
		"""

		if key is not None and not isinstance(key, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: key EXPECTED TYPE: str', None, None)
		
		if modification is not None and not isinstance(modification, int):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: modification EXPECTED TYPE: int', None, None)
		
		self.__key_modified[key] = modification



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
	from zcrmsdk.src.com.zoho.crm.api.record.record import Record
except Exception:
	from ..exception import SDKException
	from ..util import Constants
	from .record import Record


class Consent(Record):
	def __init__(self):
		"""Creates an instance of Consent"""
		super().__init__()


	def get_owner(self):
		"""
		The method to get the owner

		Returns:
			User: An instance of User
		"""

		return self.get_key_value('Owner')

	def set_owner(self, owner):
		"""
		The method to set the value to owner

		Parameters:
			owner (User) : An instance of User
		"""

		try:
			from zcrmsdk.src.com.zoho.crm.api.users import User
		except Exception:
			from ..users import User

		if owner is not None and not isinstance(owner, User):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: owner EXPECTED TYPE: User', None, None)
		
		self.add_key_value('Owner', owner)

	def get_contact_through_email(self):
		"""
		The method to get the contact_through_email

		Returns:
			bool: A bool representing the contact_through_email
		"""

		return self.get_key_value('Contact_Through_Email')

	def set_contact_through_email(self, contact_through_email):
		"""
		The method to set the value to contact_through_email

		Parameters:
			contact_through_email (bool) : A bool representing the contact_through_email
		"""

		if contact_through_email is not None and not isinstance(contact_through_email, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: contact_through_email EXPECTED TYPE: bool', None, None)
		
		self.add_key_value('Contact_Through_Email', contact_through_email)

	def get_contact_through_social(self):
		"""
		The method to get the contact_through_social

		Returns:
			bool: A bool representing the contact_through_social
		"""

		return self.get_key_value('Contact_Through_Social')

	def set_contact_through_social(self, contact_through_social):
		"""
		The method to set the value to contact_through_social

		Parameters:
			contact_through_social (bool) : A bool representing the contact_through_social
		"""

		if contact_through_social is not None and not isinstance(contact_through_social, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: contact_through_social EXPECTED TYPE: bool', None, None)
		
		self.add_key_value('Contact_Through_Social', contact_through_social)

	def get_contact_through_survey(self):
		"""
		The method to get the contact_through_survey

		Returns:
			bool: A bool representing the contact_through_survey
		"""

		return self.get_key_value('Contact_Through_Survey')

	def set_contact_through_survey(self, contact_through_survey):
		"""
		The method to set the value to contact_through_survey

		Parameters:
			contact_through_survey (bool) : A bool representing the contact_through_survey
		"""

		if contact_through_survey is not None and not isinstance(contact_through_survey, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: contact_through_survey EXPECTED TYPE: bool', None, None)
		
		self.add_key_value('Contact_Through_Survey', contact_through_survey)

	def get_contact_through_phone(self):
		"""
		The method to get the contact_through_phone

		Returns:
			bool: A bool representing the contact_through_phone
		"""

		return self.get_key_value('Contact_Through_Phone')

	def set_contact_through_phone(self, contact_through_phone):
		"""
		The method to set the value to contact_through_phone

		Parameters:
			contact_through_phone (bool) : A bool representing the contact_through_phone
		"""

		if contact_through_phone is not None and not isinstance(contact_through_phone, bool):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: contact_through_phone EXPECTED TYPE: bool', None, None)
		
		self.add_key_value('Contact_Through_Phone', contact_through_phone)

	def get_mail_sent_time(self):
		"""
		The method to get the mail_sent_time

		Returns:
			datetime: An instance of datetime
		"""

		return self.get_key_value('Mail_Sent_Time')

	def set_mail_sent_time(self, mail_sent_time):
		"""
		The method to set the value to mail_sent_time

		Parameters:
			mail_sent_time (datetime) : An instance of datetime
		"""

		from datetime import datetime

		if mail_sent_time is not None and not isinstance(mail_sent_time, datetime):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: mail_sent_time EXPECTED TYPE: datetime', None, None)
		
		self.add_key_value('Mail_Sent_Time', mail_sent_time)

	def get_consent_date(self):
		"""
		The method to get the consent_date

		Returns:
			date: An instance of date
		"""

		return self.get_key_value('Consent_Date')

	def set_consent_date(self, consent_date):
		"""
		The method to set the value to consent_date

		Parameters:
			consent_date (date) : An instance of date
		"""

		if consent_date is not None and not isinstance(consent_date, date):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: consent_date EXPECTED TYPE: date', None, None)
		
		self.add_key_value('Consent_Date', consent_date)

	def get_consent_remarks(self):
		"""
		The method to get the consent_remarks

		Returns:
			string: A string representing the consent_remarks
		"""

		return self.get_key_value('Consent_Remarks')

	def set_consent_remarks(self, consent_remarks):
		"""
		The method to set the value to consent_remarks

		Parameters:
			consent_remarks (string) : A string representing the consent_remarks
		"""

		if consent_remarks is not None and not isinstance(consent_remarks, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: consent_remarks EXPECTED TYPE: str', None, None)
		
		self.add_key_value('Consent_Remarks', consent_remarks)

	def get_consent_through(self):
		"""
		The method to get the consent_through

		Returns:
			string: A string representing the consent_through
		"""

		return self.get_key_value('Consent_Through')

	def set_consent_through(self, consent_through):
		"""
		The method to set the value to consent_through

		Parameters:
			consent_through (string) : A string representing the consent_through
		"""

		if consent_through is not None and not isinstance(consent_through, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: consent_through EXPECTED TYPE: str', None, None)
		
		self.add_key_value('Consent_Through', consent_through)

	def get_data_processing_basis(self):
		"""
		The method to get the data_processing_basis

		Returns:
			string: A string representing the data_processing_basis
		"""

		return self.get_key_value('Data_Processing_Basis')

	def set_data_processing_basis(self, data_processing_basis):
		"""
		The method to set the value to data_processing_basis

		Parameters:
			data_processing_basis (string) : A string representing the data_processing_basis
		"""

		if data_processing_basis is not None and not isinstance(data_processing_basis, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: data_processing_basis EXPECTED TYPE: str', None, None)
		
		self.add_key_value('Data_Processing_Basis', data_processing_basis)



try:
	from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
	from zcrmsdk.src.com.zoho.crm.api.util import Constants
except Exception:
	from ..exception import SDKException
	from ..util import Constants


class Field(object):
	def __init__(self, api_name):
		"""
		Creates an instance of Field with the given parameters

		Parameters:
			api_name (string) : A string representing the api_name
		"""

		if api_name is not None and not isinstance(api_name, str):
			raise SDKException(Constants.DATA_TYPE_ERROR, 'KEY: api_name EXPECTED TYPE: str', None, None)
		
		self.__api_name = api_name


	def get_api_name(self):
		"""
		The method to get the api_name

		Returns:
			string: A string representing the api_name
		"""

		return self.__api_name


	class Products(object):
		
		@classmethod
		def product_category(cls):
			return Field('Product_Category')

		@classmethod
		def qty_in_demand(cls):
			return Field('Qty_in_Demand')

		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def vendor_name(cls):
			return Field('Vendor_Name')

		@classmethod
		def tax(cls):
			return Field('Tax')

		@classmethod
		def sales_start_date(cls):
			return Field('Sales_Start_Date')

		@classmethod
		def product_active(cls):
			return Field('Product_Active')

		@classmethod
		def record_image(cls):
			return Field('Record_Image')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def product_code(cls):
			return Field('Product_Code')

		@classmethod
		def manufacturer(cls):
			return Field('Manufacturer')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def support_expiry_date(cls):
			return Field('Support_Expiry_Date')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def commission_rate(cls):
			return Field('Commission_Rate')

		@classmethod
		def product_name(cls):
			return Field('Product_Name')

		@classmethod
		def handler(cls):
			return Field('Handler')

		@classmethod
		def support_start_date(cls):
			return Field('Support_Start_Date')

		@classmethod
		def usage_unit(cls):
			return Field('Usage_Unit')

		@classmethod
		def qty_ordered(cls):
			return Field('Qty_Ordered')

		@classmethod
		def qty_in_stock(cls):
			return Field('Qty_in_Stock')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def sales_end_date(cls):
			return Field('Sales_End_Date')

		@classmethod
		def unit_price(cls):
			return Field('Unit_Price')

		@classmethod
		def taxable(cls):
			return Field('Taxable')

		@classmethod
		def reorder_level(cls):
			return Field('Reorder_Level')




	class Tasks(object):
		
		@classmethod
		def status(cls):
			return Field('Status')

		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def due_date(cls):
			return Field('Due_Date')

		@classmethod
		def priority(cls):
			return Field('Priority')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def closed_time(cls):
			return Field('Closed_Time')

		@classmethod
		def subject(cls):
			return Field('Subject')

		@classmethod
		def send_notification_email(cls):
			return Field('Send_Notification_Email')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def recurring_activity(cls):
			return Field('Recurring_Activity')

		@classmethod
		def what_id(cls):
			return Field('What_Id')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def remind_at(cls):
			return Field('Remind_At')

		@classmethod
		def who_id(cls):
			return Field('Who_Id')




	class Vendors(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def email(cls):
			return Field('Email')

		@classmethod
		def category(cls):
			return Field('Category')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def vendor_name(cls):
			return Field('Vendor_Name')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def website(cls):
			return Field('Website')

		@classmethod
		def city(cls):
			return Field('City')

		@classmethod
		def record_image(cls):
			return Field('Record_Image')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def phone(cls):
			return Field('Phone')

		@classmethod
		def state(cls):
			return Field('State')

		@classmethod
		def gl_account(cls):
			return Field('GL_Account')

		@classmethod
		def street(cls):
			return Field('Street')

		@classmethod
		def country(cls):
			return Field('Country')

		@classmethod
		def zip_code(cls):
			return Field('Zip_Code')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')




	class Calls(object):
		
		@classmethod
		def call_duration(cls):
			return Field('Call_Duration')

		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def reminder(cls):
			return Field('Reminder')

		@classmethod
		def caller_id(cls):
			return Field('Caller_ID')

		@classmethod
		def cti_entry(cls):
			return Field('CTI_Entry')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def call_start_time(cls):
			return Field('Call_Start_Time')

		@classmethod
		def subject(cls):
			return Field('Subject')

		@classmethod
		def call_agenda(cls):
			return Field('Call_Agenda')

		@classmethod
		def call_result(cls):
			return Field('Call_Result')

		@classmethod
		def call_type(cls):
			return Field('Call_Type')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def what_id(cls):
			return Field('What_Id')

		@classmethod
		def call_duration_in_seconds(cls):
			return Field('Call_Duration_in_seconds')

		@classmethod
		def call_purpose(cls):
			return Field('Call_Purpose')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def dialled_number(cls):
			return Field('Dialled_Number')

		@classmethod
		def call_status(cls):
			return Field('Call_Status')

		@classmethod
		def who_id(cls):
			return Field('Who_Id')




	class Leads(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def company(cls):
			return Field('Company')

		@classmethod
		def email(cls):
			return Field('Email')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def rating(cls):
			return Field('Rating')

		@classmethod
		def website(cls):
			return Field('Website')

		@classmethod
		def twitter(cls):
			return Field('Twitter')

		@classmethod
		def salutation(cls):
			return Field('Salutation')

		@classmethod
		def last_activity_time(cls):
			return Field('Last_Activity_Time')

		@classmethod
		def first_name(cls):
			return Field('First_Name')

		@classmethod
		def full_name(cls):
			return Field('Full_Name')

		@classmethod
		def lead_status(cls):
			return Field('Lead_Status')

		@classmethod
		def industry(cls):
			return Field('Industry')

		@classmethod
		def record_image(cls):
			return Field('Record_Image')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def skype_id(cls):
			return Field('Skype_ID')

		@classmethod
		def phone(cls):
			return Field('Phone')

		@classmethod
		def street(cls):
			return Field('Street')

		@classmethod
		def zip_code(cls):
			return Field('Zip_Code')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def email_opt_out(cls):
			return Field('Email_Opt_Out')

		@classmethod
		def designation(cls):
			return Field('Designation')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def city(cls):
			return Field('City')

		@classmethod
		def no_of_employees(cls):
			return Field('No_of_Employees')

		@classmethod
		def mobile(cls):
			return Field('Mobile')

		@classmethod
		def converted_date_time(cls):
			return Field('Converted_Date_Time')

		@classmethod
		def last_name(cls):
			return Field('Last_Name')

		@classmethod
		def layout(cls):
			return Field('Layout')

		@classmethod
		def state(cls):
			return Field('State')

		@classmethod
		def lead_source(cls):
			return Field('Lead_Source')

		@classmethod
		def is_record_duplicate(cls):
			return Field('Is_Record_Duplicate')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def fax(cls):
			return Field('Fax')

		@classmethod
		def annual_revenue(cls):
			return Field('Annual_Revenue')

		@classmethod
		def secondary_email(cls):
			return Field('Secondary_Email')




	class Deals(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def campaign_source(cls):
			return Field('Campaign_Source')

		@classmethod
		def closing_date(cls):
			return Field('Closing_Date')

		@classmethod
		def last_activity_time(cls):
			return Field('Last_Activity_Time')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def lead_conversion_time(cls):
			return Field('Lead_Conversion_Time')

		@classmethod
		def deal_name(cls):
			return Field('Deal_Name')

		@classmethod
		def expected_revenue(cls):
			return Field('Expected_Revenue')

		@classmethod
		def overall_sales_duration(cls):
			return Field('Overall_Sales_Duration')

		@classmethod
		def stage(cls):
			return Field('Stage')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def territory(cls):
			return Field('Territory')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def amount(cls):
			return Field('Amount')

		@classmethod
		def probability(cls):
			return Field('Probability')

		@classmethod
		def next_step(cls):
			return Field('Next_Step')

		@classmethod
		def contact_name(cls):
			return Field('Contact_Name')

		@classmethod
		def sales_cycle_duration(cls):
			return Field('Sales_Cycle_Duration')

		@classmethod
		def type(cls):
			return Field('Type')

		@classmethod
		def deal_category_status(cls):
			return Field('Deal_Category_Status')

		@classmethod
		def lead_source(cls):
			return Field('Lead_Source')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')




	class Campaigns(object):
		
		@classmethod
		def status(cls):
			return Field('Status')

		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def campaign_name(cls):
			return Field('Campaign_Name')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def end_date(cls):
			return Field('End_Date')

		@classmethod
		def type(cls):
			return Field('Type')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def num_sent(cls):
			return Field('Num_sent')

		@classmethod
		def expected_revenue(cls):
			return Field('Expected_Revenue')

		@classmethod
		def actual_cost(cls):
			return Field('Actual_Cost')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def expected_response(cls):
			return Field('Expected_Response')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def parent_campaign(cls):
			return Field('Parent_Campaign')

		@classmethod
		def start_date(cls):
			return Field('Start_Date')

		@classmethod
		def budgeted_cost(cls):
			return Field('Budgeted_Cost')




	class Quotes(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def discount(cls):
			return Field('Discount')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def shipping_state(cls):
			return Field('Shipping_State')

		@classmethod
		def tax(cls):
			return Field('Tax')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def deal_name(cls):
			return Field('Deal_Name')

		@classmethod
		def valid_till(cls):
			return Field('Valid_Till')

		@classmethod
		def billing_country(cls):
			return Field('Billing_Country')

		@classmethod
		def account_name(cls):
			return Field('Account_Name')

		@classmethod
		def team(cls):
			return Field('Team')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def carrier(cls):
			return Field('Carrier')

		@classmethod
		def quote_stage(cls):
			return Field('Quote_Stage')

		@classmethod
		def grand_total(cls):
			return Field('Grand_Total')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def billing_street(cls):
			return Field('Billing_Street')

		@classmethod
		def adjustment(cls):
			return Field('Adjustment')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def terms_and_conditions(cls):
			return Field('Terms_and_Conditions')

		@classmethod
		def sub_total(cls):
			return Field('Sub_Total')

		@classmethod
		def billing_code(cls):
			return Field('Billing_Code')

		@classmethod
		def product_details(cls):
			return Field('Product_Details')

		@classmethod
		def subject(cls):
			return Field('Subject')

		@classmethod
		def contact_name(cls):
			return Field('Contact_Name')

		@classmethod
		def shipping_city(cls):
			return Field('Shipping_City')

		@classmethod
		def shipping_country(cls):
			return Field('Shipping_Country')

		@classmethod
		def shipping_code(cls):
			return Field('Shipping_Code')

		@classmethod
		def billing_city(cls):
			return Field('Billing_City')

		@classmethod
		def quote_number(cls):
			return Field('Quote_Number')

		@classmethod
		def billing_state(cls):
			return Field('Billing_State')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def shipping_street(cls):
			return Field('Shipping_Street')




	class Invoices(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def discount(cls):
			return Field('Discount')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def shipping_state(cls):
			return Field('Shipping_State')

		@classmethod
		def tax(cls):
			return Field('Tax')

		@classmethod
		def invoice_date(cls):
			return Field('Invoice_Date')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def billing_country(cls):
			return Field('Billing_Country')

		@classmethod
		def account_name(cls):
			return Field('Account_Name')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def sales_order(cls):
			return Field('Sales_Order')

		@classmethod
		def status(cls):
			return Field('Status')

		@classmethod
		def grand_total(cls):
			return Field('Grand_Total')

		@classmethod
		def sales_commission(cls):
			return Field('Sales_Commission')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def due_date(cls):
			return Field('Due_Date')

		@classmethod
		def billing_street(cls):
			return Field('Billing_Street')

		@classmethod
		def adjustment(cls):
			return Field('Adjustment')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def terms_and_conditions(cls):
			return Field('Terms_and_Conditions')

		@classmethod
		def sub_total(cls):
			return Field('Sub_Total')

		@classmethod
		def invoice_number(cls):
			return Field('Invoice_Number')

		@classmethod
		def billing_code(cls):
			return Field('Billing_Code')

		@classmethod
		def product_details(cls):
			return Field('Product_Details')

		@classmethod
		def subject(cls):
			return Field('Subject')

		@classmethod
		def contact_name(cls):
			return Field('Contact_Name')

		@classmethod
		def excise_duty(cls):
			return Field('Excise_Duty')

		@classmethod
		def shipping_city(cls):
			return Field('Shipping_City')

		@classmethod
		def shipping_country(cls):
			return Field('Shipping_Country')

		@classmethod
		def shipping_code(cls):
			return Field('Shipping_Code')

		@classmethod
		def billing_city(cls):
			return Field('Billing_City')

		@classmethod
		def purchase_order(cls):
			return Field('Purchase_Order')

		@classmethod
		def billing_state(cls):
			return Field('Billing_State')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def shipping_street(cls):
			return Field('Shipping_Street')




	class Attachments(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def file_name(cls):
			return Field('File_Name')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def size(cls):
			return Field('Size')

		@classmethod
		def parent_id(cls):
			return Field('Parent_Id')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def created_by(cls):
			return Field('Created_By')




	class Price_Books(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def active(cls):
			return Field('Active')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def pricing_details(cls):
			return Field('Pricing_Details')

		@classmethod
		def pricing_model(cls):
			return Field('Pricing_Model')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def price_book_name(cls):
			return Field('Price_Book_Name')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')




	class Sales_Orders(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def discount(cls):
			return Field('Discount')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def customer_no(cls):
			return Field('Customer_No')

		@classmethod
		def shipping_state(cls):
			return Field('Shipping_State')

		@classmethod
		def tax(cls):
			return Field('Tax')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def deal_name(cls):
			return Field('Deal_Name')

		@classmethod
		def billing_country(cls):
			return Field('Billing_Country')

		@classmethod
		def account_name(cls):
			return Field('Account_Name')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def carrier(cls):
			return Field('Carrier')

		@classmethod
		def quote_name(cls):
			return Field('Quote_Name')

		@classmethod
		def status(cls):
			return Field('Status')

		@classmethod
		def sales_commission(cls):
			return Field('Sales_Commission')

		@classmethod
		def grand_total(cls):
			return Field('Grand_Total')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def due_date(cls):
			return Field('Due_Date')

		@classmethod
		def billing_street(cls):
			return Field('Billing_Street')

		@classmethod
		def adjustment(cls):
			return Field('Adjustment')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def terms_and_conditions(cls):
			return Field('Terms_and_Conditions')

		@classmethod
		def sub_total(cls):
			return Field('Sub_Total')

		@classmethod
		def billing_code(cls):
			return Field('Billing_Code')

		@classmethod
		def product_details(cls):
			return Field('Product_Details')

		@classmethod
		def subject(cls):
			return Field('Subject')

		@classmethod
		def contact_name(cls):
			return Field('Contact_Name')

		@classmethod
		def excise_duty(cls):
			return Field('Excise_Duty')

		@classmethod
		def shipping_city(cls):
			return Field('Shipping_City')

		@classmethod
		def shipping_country(cls):
			return Field('Shipping_Country')

		@classmethod
		def shipping_code(cls):
			return Field('Shipping_Code')

		@classmethod
		def billing_city(cls):
			return Field('Billing_City')

		@classmethod
		def so_number(cls):
			return Field('SO_Number')

		@classmethod
		def purchase_order(cls):
			return Field('Purchase_Order')

		@classmethod
		def billing_state(cls):
			return Field('Billing_State')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def pending(cls):
			return Field('Pending')

		@classmethod
		def shipping_street(cls):
			return Field('Shipping_Street')




	class Contacts(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def email(cls):
			return Field('Email')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def vendor_name(cls):
			return Field('Vendor_Name')

		@classmethod
		def mailing_zip(cls):
			return Field('Mailing_Zip')

		@classmethod
		def reports_to(cls):
			return Field('Reports_To')

		@classmethod
		def other_phone(cls):
			return Field('Other_Phone')

		@classmethod
		def mailing_state(cls):
			return Field('Mailing_State')

		@classmethod
		def twitter(cls):
			return Field('Twitter')

		@classmethod
		def other_zip(cls):
			return Field('Other_Zip')

		@classmethod
		def mailing_street(cls):
			return Field('Mailing_Street')

		@classmethod
		def other_state(cls):
			return Field('Other_State')

		@classmethod
		def salutation(cls):
			return Field('Salutation')

		@classmethod
		def other_country(cls):
			return Field('Other_Country')

		@classmethod
		def last_activity_time(cls):
			return Field('Last_Activity_Time')

		@classmethod
		def first_name(cls):
			return Field('First_Name')

		@classmethod
		def full_name(cls):
			return Field('Full_Name')

		@classmethod
		def asst_phone(cls):
			return Field('Asst_Phone')

		@classmethod
		def record_image(cls):
			return Field('Record_Image')

		@classmethod
		def department(cls):
			return Field('Department')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def skype_id(cls):
			return Field('Skype_ID')

		@classmethod
		def assistant(cls):
			return Field('Assistant')

		@classmethod
		def phone(cls):
			return Field('Phone')

		@classmethod
		def mailing_country(cls):
			return Field('Mailing_Country')

		@classmethod
		def account_name(cls):
			return Field('Account_Name')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def email_opt_out(cls):
			return Field('Email_Opt_Out')

		@classmethod
		def reporting_to(cls):
			return Field('Reporting_To')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def date_of_birth(cls):
			return Field('Date_of_Birth')

		@classmethod
		def mailing_city(cls):
			return Field('Mailing_City')

		@classmethod
		def other_city(cls):
			return Field('Other_City')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def title(cls):
			return Field('Title')

		@classmethod
		def other_street(cls):
			return Field('Other_Street')

		@classmethod
		def mobile(cls):
			return Field('Mobile')

		@classmethod
		def territories(cls):
			return Field('Territories')

		@classmethod
		def home_phone(cls):
			return Field('Home_Phone')

		@classmethod
		def last_name(cls):
			return Field('Last_Name')

		@classmethod
		def lead_source(cls):
			return Field('Lead_Source')

		@classmethod
		def is_record_duplicate(cls):
			return Field('Is_Record_Duplicate')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def fax(cls):
			return Field('Fax')

		@classmethod
		def secondary_email(cls):
			return Field('Secondary_Email')




	class Solutions(object):
		
		@classmethod
		def status(cls):
			return Field('Status')

		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def comments(cls):
			return Field('Comments')

		@classmethod
		def no_of_comments(cls):
			return Field('No_of_comments')

		@classmethod
		def product_name(cls):
			return Field('Product_Name')

		@classmethod
		def add_comment(cls):
			return Field('Add_Comment')

		@classmethod
		def solution_number(cls):
			return Field('Solution_Number')

		@classmethod
		def answer(cls):
			return Field('Answer')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def solution_title(cls):
			return Field('Solution_Title')

		@classmethod
		def published(cls):
			return Field('Published')

		@classmethod
		def question(cls):
			return Field('Question')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')




	class Events(object):
		
		@classmethod
		def all_day(cls):
			return Field('All_day')

		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def check_in_state(cls):
			return Field('Check_In_State')

		@classmethod
		def check_in_address(cls):
			return Field('Check_In_Address')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def start_datetime(cls):
			return Field('Start_DateTime')

		@classmethod
		def latitude(cls):
			return Field('Latitude')

		@classmethod
		def participants(cls):
			return Field('Participants')

		@classmethod
		def event_title(cls):
			return Field('Event_Title')

		@classmethod
		def end_datetime(cls):
			return Field('End_DateTime')

		@classmethod
		def check_in_by(cls):
			return Field('Check_In_By')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def check_in_city(cls):
			return Field('Check_In_City')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def check_in_comment(cls):
			return Field('Check_In_Comment')

		@classmethod
		def remind_at(cls):
			return Field('Remind_At')

		@classmethod
		def who_id(cls):
			return Field('Who_Id')

		@classmethod
		def check_in_status(cls):
			return Field('Check_In_Status')

		@classmethod
		def check_in_country(cls):
			return Field('Check_In_Country')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def venue(cls):
			return Field('Venue')

		@classmethod
		def zip_code(cls):
			return Field('ZIP_Code')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def longitude(cls):
			return Field('Longitude')

		@classmethod
		def check_in_time(cls):
			return Field('Check_In_Time')

		@classmethod
		def recurring_activity(cls):
			return Field('Recurring_Activity')

		@classmethod
		def what_id(cls):
			return Field('What_Id')

		@classmethod
		def check_in_sub_locality(cls):
			return Field('Check_In_Sub_Locality')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')




	class Purchase_Orders(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def discount(cls):
			return Field('Discount')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def vendor_name(cls):
			return Field('Vendor_Name')

		@classmethod
		def shipping_state(cls):
			return Field('Shipping_State')

		@classmethod
		def tax(cls):
			return Field('Tax')

		@classmethod
		def po_date(cls):
			return Field('PO_Date')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def billing_country(cls):
			return Field('Billing_Country')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def carrier(cls):
			return Field('Carrier')

		@classmethod
		def status(cls):
			return Field('Status')

		@classmethod
		def grand_total(cls):
			return Field('Grand_Total')

		@classmethod
		def sales_commission(cls):
			return Field('Sales_Commission')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def po_number(cls):
			return Field('PO_Number')

		@classmethod
		def due_date(cls):
			return Field('Due_Date')

		@classmethod
		def billing_street(cls):
			return Field('Billing_Street')

		@classmethod
		def adjustment(cls):
			return Field('Adjustment')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def terms_and_conditions(cls):
			return Field('Terms_and_Conditions')

		@classmethod
		def sub_total(cls):
			return Field('Sub_Total')

		@classmethod
		def billing_code(cls):
			return Field('Billing_Code')

		@classmethod
		def product_details(cls):
			return Field('Product_Details')

		@classmethod
		def subject(cls):
			return Field('Subject')

		@classmethod
		def tracking_number(cls):
			return Field('Tracking_Number')

		@classmethod
		def contact_name(cls):
			return Field('Contact_Name')

		@classmethod
		def excise_duty(cls):
			return Field('Excise_Duty')

		@classmethod
		def shipping_city(cls):
			return Field('Shipping_City')

		@classmethod
		def shipping_country(cls):
			return Field('Shipping_Country')

		@classmethod
		def shipping_code(cls):
			return Field('Shipping_Code')

		@classmethod
		def billing_city(cls):
			return Field('Billing_City')

		@classmethod
		def requisition_no(cls):
			return Field('Requisition_No')

		@classmethod
		def billing_state(cls):
			return Field('Billing_State')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def shipping_street(cls):
			return Field('Shipping_Street')




	class Accounts(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def ownership(cls):
			return Field('Ownership')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def account_type(cls):
			return Field('Account_Type')

		@classmethod
		def rating(cls):
			return Field('Rating')

		@classmethod
		def sic_code(cls):
			return Field('SIC_Code')

		@classmethod
		def shipping_state(cls):
			return Field('Shipping_State')

		@classmethod
		def website(cls):
			return Field('Website')

		@classmethod
		def employees(cls):
			return Field('Employees')

		@classmethod
		def last_activity_time(cls):
			return Field('Last_Activity_Time')

		@classmethod
		def industry(cls):
			return Field('Industry')

		@classmethod
		def record_image(cls):
			return Field('Record_Image')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def account_site(cls):
			return Field('Account_Site')

		@classmethod
		def phone(cls):
			return Field('Phone')

		@classmethod
		def billing_country(cls):
			return Field('Billing_Country')

		@classmethod
		def account_name(cls):
			return Field('Account_Name')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def account_number(cls):
			return Field('Account_Number')

		@classmethod
		def ticker_symbol(cls):
			return Field('Ticker_Symbol')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def billing_street(cls):
			return Field('Billing_Street')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def billing_code(cls):
			return Field('Billing_Code')

		@classmethod
		def territories(cls):
			return Field('Territories')

		@classmethod
		def parent_account(cls):
			return Field('Parent_Account')

		@classmethod
		def shipping_city(cls):
			return Field('Shipping_City')

		@classmethod
		def shipping_country(cls):
			return Field('Shipping_Country')

		@classmethod
		def shipping_code(cls):
			return Field('Shipping_Code')

		@classmethod
		def billing_city(cls):
			return Field('Billing_City')

		@classmethod
		def billing_state(cls):
			return Field('Billing_State')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def fax(cls):
			return Field('Fax')

		@classmethod
		def annual_revenue(cls):
			return Field('Annual_Revenue')

		@classmethod
		def shipping_street(cls):
			return Field('Shipping_Street')




	class Cases(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def email(cls):
			return Field('Email')

		@classmethod
		def description(cls):
			return Field('Description')

		@classmethod
		def internal_comments(cls):
			return Field('Internal_Comments')

		@classmethod
		def no_of_comments(cls):
			return Field('No_of_comments')

		@classmethod
		def reported_by(cls):
			return Field('Reported_By')

		@classmethod
		def case_number(cls):
			return Field('Case_Number')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def deal_name(cls):
			return Field('Deal_Name')

		@classmethod
		def phone(cls):
			return Field('Phone')

		@classmethod
		def account_name(cls):
			return Field('Account_Name')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def solution(cls):
			return Field('Solution')

		@classmethod
		def status(cls):
			return Field('Status')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def priority(cls):
			return Field('Priority')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def comments(cls):
			return Field('Comments')

		@classmethod
		def product_name(cls):
			return Field('Product_Name')

		@classmethod
		def add_comment(cls):
			return Field('Add_Comment')

		@classmethod
		def case_origin(cls):
			return Field('Case_Origin')

		@classmethod
		def subject(cls):
			return Field('Subject')

		@classmethod
		def case_reason(cls):
			return Field('Case_Reason')

		@classmethod
		def type(cls):
			return Field('Type')

		@classmethod
		def is_record_duplicate(cls):
			return Field('Is_Record_Duplicate')

		@classmethod
		def tag(cls):
			return Field('Tag')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def related_to(cls):
			return Field('Related_To')




	class Notes(object):
		
		@classmethod
		def owner(cls):
			return Field('Owner')

		@classmethod
		def modified_by(cls):
			return Field('Modified_By')

		@classmethod
		def modified_time(cls):
			return Field('Modified_Time')

		@classmethod
		def created_time(cls):
			return Field('Created_Time')

		@classmethod
		def parent_id(cls):
			return Field('Parent_Id')

		@classmethod
		def id(cls):
			return Field('id')

		@classmethod
		def created_by(cls):
			return Field('Created_By')

		@classmethod
		def note_title(cls):
			return Field('Note_Title')

		@classmethod
		def note_content(cls):
			return Field('Note_Content')


